#
# $Id: stamp.py,v 1.10 2006/05/19 23:50:35 mwalfish Exp $
#   utilities for working with stamps. stamps are basically just a
#   certificate and a disposable. the disposable is single-use.
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import dqm.err as dqm_err
import dqm.crypt.cert as dqm_cert
import dqm.crypt.disposable as disposable
import dqm.constants
import binascii

class BaseStamp:

    def to_str(self, cert_str, disposable_str):
	raise AssertionError, 'Abstract method called'

    def verify(self, stamp_str, signers_db):
	raise AssertionError, 'Abstract method called'

# stamp, logical version:
#   Version
#   Cert 
#   Disposable
#
# stamp, stringized version:
#   version#signer_id:pub_key:quota:expiry:sig#i:weekstamp:sig 
#    <all binary values are base64 encoded>
#
# DHT val:
#    DQM_MAGIC_STR + hash(stamp_stringized)
#
# DHT key:
#   postmark = hash(DHT val)
class SimpleStamp(BaseStamp):

    DELIM = '#'

    def to_str(self, cert_str, disposable_str):
	
	DELIM = SimpleStamp.DELIM
	ver_str = dqm.constants.VERSION
	stamp_str = DELIM.join([ver_str, cert_str, disposable_str])
	return stamp_str

    # signers_db is a dictionary mapping trusted signer ids to public keys
    def verify(self, stamp_str, signers_db):

	DELIM = SimpleStamp.DELIM
	try: 
	    ver,cert_str,disposable_str = stamp_str.split(DELIM)
	    cert = dqm_cert.SimpleCert().str_to_cert(cert_str)
	    disp = \
	    disposable.SimpleDisposable().str_to_disposable(disposable_str)
	except (ValueError, binascii.Error): return dqm_err.WRONG_FORMAT

	# check that the stamp verifier trusts the signer of the
	# certificate
	signer_id = cert['signer_id']
	if signers_db.has_key(signer_id) is False:
	    return dqm_err.INVALID_CERT_SIGNER

	cert_signer_pubkey = signers_db[signer_id]
	rc = dqm_cert.SimpleCert().verify(cert_str, cert_signer_pubkey)
	if rc != dqm_err.SUCCESS: return rc	

	rc = disposable.SimpleDisposable().verify(disposable_str, 
						  cert['pub_key'])
	if rc != dqm_err.SUCCESS: return rc

	quota = cert['quota']
	curr  = disp['i']

	if (curr < quota): return dqm_err.SUCCESS
	else: return dqm_err.OVER_QUOTA
