#
# $Id: db.py,v 1.12 2006/05/19 23:50:35 mwalfish Exp $ 
#  
#  have the disposables be backed by an on-disk db. better recovery
#  semantics and presumably better performance than the brain-dead 'simple'
#  module
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import os
import os.path
import random
import base
import bsddb3.db as bdb
from dqm.util.convert import *

ENV_FL = bdb.DB_INIT_LOCK  | bdb.DB_INIT_TXN  | bdb.DB_INIT_LOG |\
         bdb.DB_INIT_MPOOL | bdb.DB_RECOVER   

def create_new(quota, d, name):
    """creates a DBPool on disk; returns a DBPool object. If there was
    previously a DBPool in directory 'd', it will be deleted."""

    # delete what was there before
    if os.path.exists(d): 
	for f in os.listdir(d): os.remove(os.path.join(d,f))
    else: 
	os.mkdir(d)
    
    db_env, db = open_db(d, name)
    return DBPool(db_env, db, quota)

def create_from_file(d, name):
    """returns a DBPool that is read from a file. The DBPool must already
    exist."""
    if not os.path.exists(d):
	raise ValueError, "DB directory doesn't exist"

    db_env, db = open_db(d, name)
    return DBPool(db_env, db)

def open_db(d, name):

    create_flag = bdb.DB_CREATE

    db_env = bdb.DBEnv()
    db_env.open(d, ENV_FL | create_flag)
    db = bdb.DB(db_env)

    tn = db_env.txn_begin()
    db.open(name, dbtype=bdb.DB_BTREE, flags=create_flag, txn=tn)
    tn.commit()

    return db_env, db


class DBPool(base.BasePool):

    # XXX: if we need to speed up the DB, then have two DBs, 
    # one for reading and the other for storing the current pointer.
    # in the read-only DB, we just keep calling db.next().
    # if we could find a pseudo-random sequence to replace the read-only
    # DB, that would be even better
    def __eq__(self,other):
	return self.db == other.db

    def __init__(self, db_env, db, quota=None):

	base.BasePool.__init__(self,quota)
	self.db_env = db_env
	self.db     = db
	self.closed = False

	if quota is not None:

	    tn = self.db_env.txn_begin()

	    self.db.put('quota',   long_to_blob(quota), txn=tn)
	    self.db.put('current', long_to_blob(0),     txn=tn)
	    
	    # XXX: random.shuffle is too slow. fix!
	    seq = range(quota)
	    random.shuffle(seq)

	    # construct a dictionary from the random shuffle. note that
	    # we store the dictionary keys as python byte arrays in
	    # big-endian order. the purpose of the python byte array is
	    # that the keys to the berkeley DB are required to be strings.
	    # the purpose of the big-endian order is that we want a
	    # the sorted order of the keys (which happens via a string
	    # sort) to come out the same as a numerical sort of the keys
	    d = dict([(long_to_blob(k),long_to_blob(v)) for \
			  k,v in enumerate(seq)])

	    for key in d.keys(): self.db.put(key, d[key], txn=tn)

	    tn.commit()

	self.quota = to_long(self.db.get('quota'))
	self.curr  = to_long(self.db.get('current'))

    def __del__(self):
	if self.closed is False: self.close()

    def close(self):
	self.db.close()
	self.db_env.close()
	self.closed = True

    def num_left(self):
	return self.quota - self.curr

    def get_quota(self):
	return self.quota

    def consume_disposable(self):
	
	if self.curr == self.quota: return -1

	tn = self.db_env.txn_begin()	

	key = long_to_blob(self.curr)	
	val = to_long(self.db.get(key, txn=tn))

	self.curr = self.curr + 1
	self.db.put('current', long_to_blob(self.curr), txn=tn)

	tn.commit()

	return val
