#
# $Id: sndr.py,v 1.9 2006/05/19 23:50:35 mwalfish Exp $
# 
# Top-level functions executed by stamp senders
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import os.path
import dqm.stamp_pool.db    as dqm_pool
import dqm.crypt.stamp      as dqm_stamp
import dqm.crypt.disposable as dqm_disposable
import dqm.cryptkey_wrap.cryptkey as cryptkey
import dqm.logger

class Sender:

    def __init__(self, which, config_obj, logger=None):

	if logger is None: logger = dqm.logger.stderr_logger()
	self.logger  = logger

	assert(which == 'stamp')

	self.pool_dir, self.pool_file \
		       = os.path.split(config_obj['stamp_pool'])
	cert_file      = config_obj['certificate_file']
	privkey_file   = config_obj['privkey_file']
	
	self.which = which
	self.dpool = dqm_pool.create_from_file(self.pool_dir, self.pool_file)

	def log_failed_file(fname,e):
	    msg = "could not open file " + fname + " Error = " + str(e)
	    logger.log(msg)

	try: 
	    f = file(cert_file, 'rb')
	    self.cert_str = f.read()
	    f.close()
	    self.cert_str = self.cert_str.strip()

	except IOError,e:   
	    log_failed_file(cert_file,e)
	    raise 

	self.privkey = cryptkey.file_to_privkey(privkey_file)
	if self.privkey is None:
	    log_failed_file(privkey_file, "")
	    raise IOError

    def construct_stamp(self):
	"""This method constructs a stamp and returns it. It also handles
	the details of the bookkeeping, i.e., crossing out the used
	'diposable' on the disk. If we're out of disposables, then the
	empty string is returned."""

	if self.dpool.num_left() == 0: return ""

	# 1. figure out which disposable we're using
	# 2. delete the disposable from the pool and write the pool to disk
	# 3. use the disposable
	# (we do things in that order in case a crash happens between steps
	# 2. and 3. -- the on-disk store should always reflect a valid set
	# of disposables)
	i = self.dpool.consume_disposable()

	logmsg = 'allocating disposable #%d from %s pool' % (i, self.which)
	self.logger.log(logmsg)

	disposable_wrapper = dqm_disposable.SimpleDisposable()
	disposable_str = disposable_wrapper.create(i, self.privkey)

	stamp_wrapper = dqm_stamp.SimpleStamp()
	stamp_str = stamp_wrapper.to_str(self.cert_str, disposable_str)

	return stamp_str
