#! /bin/env python
#
# $Id: stamp-printer.py,v 1.4 2006/05/19 23:50:35 mwalfish Exp $
#
#  simple command-line app to demonstrate how to test the sender
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import sys
import getopt
import textwrap
import dqm.config.user as dqm_config
import dqm.user.sndr as dqm_sndr
import dqm.logger

if __name__ == "__main__":
    """This is a simple command-line app to create a stamp. The stamp
    is printed on stdout"""

    def usage():
	print """usage: %s [-l logfile] [-w] [-m] config-file
	   creates a stamp using the pool of disposables listed in
	   config-file. 
	    -l option specifies a logfile. if none is given, log messages
	       are printed to stderr.
	    -w option to specify text wrapping. if given, stamps are not
	       printed as one continuous string but are instead wrapped to
	       comply with e-mail standards that say that lines should
	       be limited to 78 characters.""" % sys.argv[0]

    def usage_and_exit():
	usage()
	sys.exit(2)

    logfile = None
    wrap    = False
    which   = 'stamp'

    try:
	opts, args = getopt.getopt(sys.argv[1:], "l:wm")
    except getopt.GetoptError:
	usage_and_exit()

    if len(args) != 1: usage_and_exit()

    for o,a in opts:
	if o == '-l':
	    logfile = a
	if o == '-w':
	    wrap = True

    configfile = args[0]

    config_obj = dqm_config.parse_config(configfile)

    if logfile is None: logger = dqm.logger.stderr_logger()
    else:               logger = dqm.logger.file_logger(sys.argv[2])

    Sndr = dqm_sndr.Sender(which, config_obj, logger)

    stamp_str = Sndr.construct_stamp()
    if stamp_str == "": 
	logger.log("no stamps to construct. not attaching any.")
	stamp_str = "OUT OF STAMPS."

    if not wrap: 
	print stamp_str
    else:
	print textwrap.fill(stamp_str, 
		    expand_tabs=False, 	
		    replace_whitespace=False,
		    subsequent_indent='\t')

