#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import struct
import binascii
import sha

pack    = struct.pack
unpack  = struct.unpack
b2a_hex = binascii.b2a_hex

ID_SIZE = sha.digest_size

def id2long(id):
    return bytes2long(id)

def long2id(n):
    return long2bytes(n,ID_SIZE)


# The next two functions are somewhat copied from the file Util/number.py in 
# the "Python Cryptography Toolkit". That file claims that this is the 
# fastest way the author has found to perform these functions. I'm adding
# to long2bytes to handle the padding in a better way.

# Convert from a blob, represented as a python byte array (a/k/a
# a python string) to a python long (a/k/a a looooooong number). 
def bytes2long(s):

    acc    = 0L
    length = len(s)
    
    # add 0s to the beginning of the byte string. these 0s are going in what 
    # are going to be interpreted as the high order bytes.
    if length % 4:
	extra = (4 - length % 4)
	s = '\000' * extra + s
	length = length + extra

    for i in range(0, length, 4):
	acc = (acc << 32) + unpack('>I', s[i:i+4])[0]

    return acc


# Perform the conversion that is the inverse of the one above.
# If 'bytes' is supplied, the return value will be padded with 0s, 
# if necessary. It is an error to call this function with bytes>0 with
# a number n that actually requires more than 'bytes' bytes.
def long2bytes(n, bytes=0):

    s = ''
    n = long(n)
    
    while n > 0:
	# grab the bottom 32 bits
	s = pack('>I', n & 0xffffffffL) + s
	n = n >> 32

    if (bytes > 0):
	# handle padding right
	diff  = bytes - len(s)
	if diff > 0:
	    s = '\000' * diff + s
	elif diff < 0:
	    # We have 'diff' too many bytes, so remove diff
	    # bytes from the beginning of the string. Note that
	    # this may truncate the number, but if that happens, it's
	    # a caller error
	    s = s[diff:]
    else:
	# the caller doesn't care how many bytes are returned, so strip off 
	# leading zeroes.
	for i in range(len(s)):
	    if s[i] != '\000':
		break
	else:
	    # only happens when n == 0
	    s = '\000'
	    i = 0
	s = s[i:]

    return s

# wrappers for unsightly syntax
def long_to_blob(l):
    return pack(">L", l)

def to_long(str):
   return unpack(">L", str)[0]


def to_byte(str):
    return unpack("B", str)[0]

def to_short(str):
    return unpack(">H", str)[0]

def to_hex(str):
    return binascii.b2a_hex(str)
