#!/bin/env python
#
# $Id: testcase.py,v 1.3 2006/05/19 23:50:35 mwalfish Exp $ 
#  
#  test routine for the berkeley DB transaction subsystem. find a way to
#  wrap this stuff in test cases and exit with the wrong codes if anything
#  goes wrong.
#
#
# Copyright (C) 2005-2006 Michael Walfish
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
import os
import sys
import time
import bsddb3.db as bdb

EXTENT_SIZE = 16*1024
LOCK_NAME   = "asdf_cancel_pool_lock"
ENV_NEW_FL  = bdb.DB_INIT_LOCK | bdb.DB_INIT_TXN  | bdb.DB_INIT_MPOOL |\
	      bdb.DB_CREATE | bdb.DB_RECOVER | bdb.DB_INIT_LOG
ENV_JOIN_FL = bdb.DB_JOINENV
DB_FLAGS    = bdb.DB_CREATE
DB_REC_LEN  = 2048

DB_MODE	    = 0600 
DB_DIR	    = "db_dir"   
DB_NAME	    = "test_db"
SLEEP_INT   = 5 

def output_err(errval):
    rc,msg = errval
    print "error: " + msg + ". code: " + str(rc)

def produce(db, num, txn):

    for i in range(num):

	while True:

	    try: 
		ret = db.append(str(i), txn)
		print ret
		break

	    except bdb.DBLockDeadlockError, errval: 
		print "PISS"
		output_err(errval)
		txn.abort()
		return False

	    time.sleep(SLEEP_INT)

    return True

def consume(db, num, txn):

    i = 0

    while i < num:

	while True:

	    try: 
		tup = db.consume(txn=txn, flags=bdb.DB_RMW)
		if (tup is not None):
		    print tup
		    break

	    except bdb.DBLockDeadlockError, errval:
		print "BUMMER"
		output_err(errval)
		txn.abort()
		return False

	    time.sleep(SLEEP_INT)

	i = i + 1

    return True

if __name__ == "__main__":
  
    def usage():
	print """usage: %s p|c num""" % sys.argv[0]

    def usage_and_exit():
	usage()
	sys.exit(2)

    if (len(sys.argv) != 3):
	usage_and_exit()

    if (sys.argv[1] not in ['p', 'c']):
	usage()

    which = sys.argv[1]
    num   = int(sys.argv[2])

    func_table = { 'p' : produce, 'c' : consume }

    is_new = False 

    # let's make sure DB_DIR exists. if not, create it.
    if (os.access(DB_DIR, os.F_OK) != 1): 
	is_new = True
	os.mkdir(DB_DIR)

    flags = { True: ENV_NEW_FL, False: ENV_JOIN_FL }[is_new]

    db_env = None; db = None;

    try:

        db_env = bdb.DBEnv()
        db_env.open(DB_DIR, flags, DB_MODE)

	db = bdb.DB(db_env)
        db.set_re_len(DB_REC_LEN)
        db.set_q_extentsize(EXTENT_SIZE)
	# NOTE: later transactions will fail if we don'txn surround the DB
	# open with a transaction. 
	txn = db_env.txn_begin()
	db.open(DB_NAME, dbtype=bdb.DB_QUEUE, flags=DB_FLAGS, txn=txn)
	txn.commit()

	txn = db_env.txn_begin()
	rc = func_table[which](db, num, txn)
	if rc is True: 
	    pass;
	    txn.commit()
    
	db.close()
	db_env.close()

    except bdb.DBRunRecoveryError, errval: 
	rc,msg = errval
	print "catastrophic error occurred: " + msg + ". code: " + str(rc)
	print "need to exit the application."


    del db
    del db_env
