/* 
 * test case to demonstrate strange behavior of DB locking function.
 *   Michael Walfish, 16 Feb. 2004
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <db.h>
#include <assert.h>
#include <string.h>

#define OUR_DB_DIR	      "db_dir"
#define OUR_DB_NAME	      "test_db"
#define OUR_DBENV_NEW_FLAGS   (DB_INIT_LOCK | DB_INIT_MPOOL | DB_INIT_TXN |\
			       DB_CREATE | DB_INIT_LOG | DB_RECOVER)
#define OUR_DBENV_JOIN_FLAGS  (DB_JOINENV)
#define OUR_DB_FLAGS          (DB_CREATE)
#define OUR_DB_REC_LEN	      4

#define ERR_CHECK(rc, ckpt)    \
    if (rc != 0) { \
	printf("error. ckpt: %s. rc: %d. msg: %s.\n", \
	       ckpt, rc, db_strerror(rc)); \
	exit(0); \
    } \

/* 
 * simple program -- the intent is to run two of these at once to see
 * whether the locker does what it is supposed to (namely letting the
 * second locker proceed only after the first locker has released the
 * lock).
 */
int main(int argc, char** argv)
{
    int flags, is_new, rc;
    int do_recovery;
    DB_ENV* dbe;
    DB* db;
    DB_TXN* tid;

    DBT thang;
    char* lock_name = "some_name_for_a_lock";

    thang.data = lock_name;
    thang.size = strlen(lock_name);

    do_recovery = (argc > 1);

    is_new = ! mkdir(OUR_DB_DIR, 0777);
    
    rc = db_env_create(&dbe, 0);
    dbe->set_errfile(dbe, stdout);
    ERR_CHECK(rc, "a")

    /* 
     * NOTE: if the directory already exists, we can join the environment. 
     * Otherwise, we have to make the environment.
     */
    if (do_recovery || is_new) flags = OUR_DBENV_NEW_FLAGS;
    else	               flags = OUR_DBENV_JOIN_FLAGS;

    rc = dbe->open(dbe, OUR_DB_DIR, flags, 0); 

    ERR_CHECK(rc, "b")
    rc = db_create(&db, dbe, 0);
    ERR_CHECK(rc, "c")

    rc = dbe->txn_begin(dbe, NULL, &tid, 0);
    ERR_CHECK(rc, "x")
    rc = db->open(db, tid, OUR_DB_NAME, NULL, DB_QUEUE, OUR_DB_FLAGS, 0);
    ERR_CHECK(rc, "e")
    rc = tid->commit(tid, 0);
    ERR_CHECK(rc, "y")

    printf("about to begin transaction.\n");
    rc = dbe->txn_begin(dbe, NULL, &tid, 0);
    ERR_CHECK(rc, "g")
    printf("begun transaction\n");

    sleep(10);

    printf("about to commit transaction.\n");
    rc = tid->commit(tid, 0);
    ERR_CHECK(rc, "h")
    printf("committed transaction\n");

    exit(0);
}
