/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "parseopt.h"
#include "config.h"
#include "rxx.h"
#include "util.h"
#include "dqm_node.h"

config_params::config_params(str conf_file) :
  rd_size(-1), wr_size(-1), nrings(-1), d(-1), 
  cache_entries(1), nindex(0), repl_factor(-1),  
  using_tree(false)
{
    parseargs pa(conf_file);
    int line;
    vec<str> av;
    conftab ct;
    bool errors = false;

    ct.add("read_size",       &rd_size,  LINUX_ALIGN, (1 << 29));
    ct.add("write_size",      &wr_size,  LINUX_ALIGN, (1 << 29));
    ct.add("portal_port",     &portal_prt, 0, 65535);
    ct.add("storer_port",     &storer_prt, 0, 65535);
    ct.add("internal_port",   &internal_prt, 0, 65535);
    ct.add("key_val_storage", &key_val_store);
    ct.add("identity",        &iden);
    ct.add("num_rings",       &nrings, 0, ~(1 << 31));
    ct.add("tree_degree",     &d, 0, ~(1 << 31));
    ct.add("r", &repl_factor, 1, (1 << 29));
    ct.add("cache_entries", &cache_entries, 1, (1 << 30));
    ct.add("index_entries", &nindex, 0, ~0);
    ct.add("storer",          wrap(this, &config_params::process_storer));
    ct.add("use_tree", &using_tree);
    ct.ignore("whos_approved_list");
    ct.ignore("trusted_signer");

    while (pa.getline(&av, &line)) {
	if (!ct.match(av, conf_file, line, &errors)) {
	    warn << conf_file << ":" 
	         << line << ": unknown config parameter\n";
	    errors = true;
	}
	if (errors) 
	    fatal << "error in config file\n";
    }

    if (repl_factor == -1)
	fatal << "must specify value for 'r' in config file\n";

}

config_params::~config_params()
{
}

void
config_params::parse_double(const char* w, 
			    double* d, vec<str> s, str loc, bool* errp)
{
    if (s.size() != 2) {
	warn << loc << ": usage: " << w << " <double>\n";
	*errp = true;
	return;
    }

    *errp = convertdouble(s[1], d);
    if (*errp) 
	warn << loc << ": usage: " << w << " <double>\n";
}

void 
config_params::process_storer(vec<str> s, str loc, bool* errp)
{
    static char* usagemsg = ": usage: storer node_name"
                            " hostname:storer_port:internal_port\n";
    if (s.size() != 3) {
	warn << loc << usagemsg;
	*errp = true;
	return;
    }
    
    static rxx colon(":");
    vec<str> vals;
    split(&vals, colon, s[2].cstr());

    if (vals.size() != 3) {
	warn << loc << usagemsg;
	*errp = true;
	return;
    }

    storer_info storer;
    storer.hostname      = vals[0];
    storer.storer_port   = atoi(vals[1].cstr());
    storer.internal_port = atoi(vals[2].cstr());

    // XXX: move to async?
    struct hostent* he = gethostbyname(storer.hostname.cstr());
    if (he == NULL) 
	fatal("DNS lookup failed for host %s\n", storer.hostname.cstr());

    memcpy((char*)&storer.ip_addr,
	   he->h_addr_list[0],
	   sizeof(u_int32_t));

    storers.insert(s[1], storer);
}
