/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _DATE_H_
#define _DATE_H_

#include <time.h>
#include "async.h"

struct date { 
   
   short year;
   char month;
   char day;

   str to_str();

   date() : year(0), month(0), day(0) {}

   date(short year, char month, char day) : 
	year(year), month(month), day(day) {}

   date(const struct tm* t) :
	year(t->tm_year + 1900),
	month(t->tm_mon + 1),
	day(t->tm_mday) {}

   static date today();
   static date today_local();
};

// does the dirty work of calling strtok and strtol and doing error
// checking
inline int
inttok(char* s, char* delim)
{
    int ret;
    char* errp;

    char* f = strtok(s, delim); 
    if (f != NULL) {
	ret = strtol(f, &errp, 10);
	if (*errp) { 
	    ret = -1;
	}
    } else {
	ret = -1;
    } 
    
    return ret;
}

// assumes date is stored in YYYY-MM-DD format
inline date
str2date(str& s, bool* err)
{
    *err = true;

    mstr ds(s.len());
    strcpy(ds.cstr(), s.cstr());

    int year, month, day;
    if ( (year = inttok(ds.cstr(), "-")) == -1) return date(0,0,0);
    if ( (month = inttok(NULL, "-"))    == -1) return date(0,0,0);
    if ( (day = inttok(NULL, "-"))      == -1) return date(0,0,0);
    
    // should be nothing left after the first three tokens have been
    // consumed
    if (strtok(NULL, "-")) return date(0,0,0);

    *err = false;
    return date(year, month, day);
}

inline str
date::to_str()
{
    strbuf sb;
    sb << year << "-" << month << "-" << day;
    return sb;
}

inline date
date::today()
{
    time_t t;
    time(&t);
    struct tm* tm_now = gmtime(&t);

    return date(tm_now);
}

inline date
date::today_local()
{
    time_t t;
    time(&t);
    struct tm* tm_now = localtime(&t);

    return date(tm_now);
}

inline bool
operator== (const date& d1, const date& d2) 
{
    return (d1.day == d2.day     && 
            d1.month == d2.month &&
	    d1.year == d2.year);

}

inline bool
operator!= (const date& d1, const date& d2)
{
    return (d1.day != d2.day     ||
            d1.month != d2.month ||
	    d1.year != d2.year);
}

// returns:
//  -1 if d1 < d2
//   0 if d1 == d2
//   1 if d1 > d2
inline int
datecmp (const date& d1, const date& d2)
{
    if (d1.year < d2.year)
	return -1;

    if (d1.year > d2.year)
	return 1;

    if (d1.month < d2.month)
	return -1;

    if (d1.month > d2.month)
	return 1;

    if (d1.day < d2.day)
	return -1;

    return (d1.day > d2.day);
}

inline bool
operator < (const date& d1, const date& d2) 
{
    return (datecmp(d1,d2) < 0);
}

inline bool
operator <= (const date& d1, const date& d2) 
{
    return (datecmp(d1,d2) <= 0);
}

inline bool
operator > (const date& d1, const date& d2) 
{
    return (datecmp(d1,d2) > 0);
}

inline bool
operator >= (const date& d1, const date& d2) 
{
    return (datecmp(d1,d2) >= 0);
}

#endif
