/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include <time.h>
#include <stdio.h>
#include "async.h"

class base_logger {

    public:
        virtual void log(const char*) = 0;
	virtual void sync() {}
	virtual ~base_logger() {}
};

class null_logger {

    public:
        virtual void log(const char*) {}
	virtual ~null_logger() {}

};

class stderr_logger : public base_logger {

    public:
        virtual void log(const char* msg) { warnx << msg << "\n"; }

};

class file_logger : public base_logger {

    public:

	file_logger(const char* logname);
	virtual ~file_logger();

	int   get_fd()      { return fd;      }
    
	virtual void log(const char* msg);

	virtual void sync();

    private:

	int fd;
};
