/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _LRU_CACHE_H_
#define _LRU_CACHE_H_

#include "async.h"
#include "keyfunc.h"
#include "ihash.h"
#include "list.h"
#include "dqm_node.h"

// based on Maxwell's LRU cache. some small changes.

struct kv_rec {
    kv_rec(ptr<dqm_key> k, ptr<dqm_val> v) : key(k), val(v) {}
    ptr<dqm_key> key;
    ptr<dqm_val> val;
    tailq_entry<kv_rec> qlink;
    ihash_entry<kv_rec> hlink;
};

class lru_cache {

    public: 
	lru_cache(int max_blocks) : max_size(max_blocks), curr_size(0) {}
	~lru_cache() { map.traverse( wrap(this, &lru_cache::remove)); }

	kv_rec* lookup(ptr<dqm_key>, bool do_touch);
	void remove(kv_rec*);
	bool add(kv_rec*); // returns 'false' if entry wasn't accepted
	void add(ptr<dqm_key> k, ptr<dqm_val> v); 
	kv_rec* oldest() { return q.first; }
	size_t size() { assert(curr_size == map.size()); return curr_size; }
	
	/* debug */
	void dump();

    private:
	
	void touch(kv_rec*);

	size_t max_size;
	size_t curr_size;

	tailq<kv_rec, &kv_rec::qlink> q;
	// could move to fhash if we didn't want to store ptr<dqm_key>
	// and instead were willing to recompute it from the val. fhash
	// is for when the key is implicit ("function-hash", presumably)
	ihash<ptr<dqm_key>, kv_rec, &kv_rec::key, &kv_rec::hlink> map;
};


#endif
