/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _NODETABLE_GEOM_H_
#define _NODETABLE_GEOM_H

#include "nodetable.h"

#define MODULUS   (1 << 28)

// First implementation of node table in C++. For now, just implementing 
// the scheme we were using in Python at the time of the SOSP submission.
// There are many virtual rings (to help with load balance b/c with only
// one ring, there is logarithmic imbalance), and a given key randomly picks a 
// ring and then is spaced at geometrically decreasing distances between the 
// local node and the target node.
class nodetable_geom : public nodetable {

    public:

	nodetable_geom(double dee = 2, int num_rings = 64);
	~nodetable_geom();

	ptr<node_list> calc_path_get(ptr<dqm_key> k) { return calc_path(k); }
	ptr<node_list> calc_path_put(ptr<dqm_key> k) { return calc_path(k); }
	ptr<node_list> calc_path_put(ptr<node_list> l) { return l; }

	void dump();

	void set_local_node(const node_id&);

	void add_node      (const node_id&, const str& alias_seed);
	void remove_node   (const node_id&);

	status get_state(const node_id& id)  
	  {  return *(nodes[id]); }

	void set_state(const node_id& id, status stat)  
	  { *(nodes[id]) = stat;  }

	void output_params(ptr<base_logger>)  {}

    private:
	double d;
	u_int32_t nrings;
	u_int32_t nnodes;
	vec<double> powers_of_d;

	// core data structures to hold the node table info
	
	// maps each node's canonical id to 'UP' or 'DOWN'
	qhash<node_id, status> nodes;

	// a set of rings (this is the 'many rings' approach that David
	// suggested). each ring contains a list of aliases (that are used
	// to order the ring) with their associated ids.
	vec<ring_lst*> rings;
	
	// maps a node to its set of aliases
	qhash<node_id, alias_vec> node_aliases;

	// need to know the local node because that's how the candidate
	// nodes are selected.
	node_id local_id;

	double stopping_ratio;

	// helper functions
	ptr<node_list> calc_path(ptr<dqm_key>);
	ringmember_lstelm* get_succ(node_alias, u_int32_t which_ring);
	u_int32_t choose_ring(const dqm_key&);
	node_alias hash2alias(const u_char* hash);
};

#endif
