/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _NODETABLE_NOTREE_H_
#define _NODETABLE_NOTREE_H_

#include "async.h"
#include "nodetable.h"

// The no-tree node table was suggested by David. The idea is that every
// key picks $k$ nodes to live on. PUTs and GETs use those $k$ nodes. The
// protocol we will start by implementing is "PUT to 1 node; GET from the
// $k$ nodes in sequence, until one of them gives an answer".

class nodetable_notree : public nodetable {

  public:
    nodetable_notree(int r, u_int32_t naliases=50);
    ~nodetable_notree();
    
    ptr<node_list> calc_path_get(ptr<dqm_key> key)
	{ return r_nodes(key); }

    ptr<node_list> calc_path_put(ptr<dqm_key> key) { 
	ptr<node_list> full_path = r_nodes(key);
	return calc_path_put(full_path);
    }

    ptr<node_list> calc_path_put(ptr<node_list>);

    void dump();

    void set_local_node(const node_id&);

    void add_node      (const node_id&, const str& alias_seed);
    void remove_node   (const node_id&);

    status get_state(const node_id& id)  
      {  return *(nodes[id]); }

    void set_state(const node_id& id, status stat)  
      { *(nodes[id]) = stat;  }

    void output_params(ptr<base_logger>);

  private:

    u_int32_t naliases;
    double    p;
    int       r;
    int       nnodes;

    // Main data structure is a list with all of the nodes' aliases

    // map each node's permanent id to 'UP' or 'DOWN'
    qhash<node_id, status>    nodes;

    // maps each node to its group of aliases. each node needs many aliases
    // so that each node will be responsible for an approximately even
    // piece of identity space. if each node has only one alias, then the
    // imbalance is logarithmic.
    qhash<node_id, alias_vec> node_aliases;

    // a list of all of the aliases and their associated ids.
    ring_lst lst;

    node_id local_id;

    // helper functions
    void recalc_k();
    bool in_vec(node_id id, ptr<node_list> vec);
    node_alias h_i_as_alias(const char* p, u_int32_t len, int i);

    ptr<node_list> r_nodes(ptr<dqm_key>);  // workhorse
    node_alias hash2alias(const u_char* hash);

};

#endif
