/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _STORER_H_
#define _STORER_H_

#include "async.h"
#include "dqm_node.h"

class svccb;
class asrv;
class nodetable;
class base_logger;
class counters;
class ondisk_storage;
class lru_cache;

class storer : public virtual refcount {

    public:
	storer(u_int16_t storer_port, 
	       ptr<nodetable>, 
	       ptr<ondisk_storage>,
	       ptr<lru_cache>, ptr<base_logger>);
	~storer();

	void dispatch(svccb*);
	void stop();

	void set_counters(counters*);

	void local_put(ptr<dqm_key>, ptr<dqm_val>);
	void local_get(ptr<dqm_key>, getcb cb);

    private:

	void handle_NULL(svccb*);
	void handle_PUT(svccb*);
	void handle_GET(svccb*);

	void put_worker(ptr<dqm_key>, ptr<dqm_val>);

	void get_worker(ptr<dqm_key>, getcb);
	void get_worker_2(ptr<dqm_key> key, getcb, ptr<dqm_val>);
	void complete_GET(svccb*, ptr<dqm_val> val);

	ptr<asrv>        srv;
	ptr<nodetable>   nodetbl;
	ptr<lru_cache>   cache;
	ptr<ondisk_storage> storage;
	ptr<base_logger> logger;
	counters*        cntrs;
};

#endif
