/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include <stdlib.h>
#include <stdio.h>
#include "storer.h"

int main(int argc, char** argv)
{
    int rc;
    const char* key1 = "abcdefghij";
    const char* val1 = "this_is_a_test";
    const char* key2 = "zzzxcsdf";

    char ret_val[1024];

    if (argc != 2) {
	printf("usage: %s db_dir\n", argv[0]);
	exit(1);
    }

    dqm_db db(argv[1], "dqm_db");

    db.put(key1, strlen(key1), val1, strlen(val1));

    u_int32_t vsize = sizeof(ret_val);
    bzero(ret_val, sizeof(ret_val));
    rc = db.get(key1, strlen(key1), ret_val, &vsize);

    // make sure the rc reports that the value is returned
    if (rc != 0) {
	printf("rc: %d\n", rc);
	assert(false);
    }

    // make sure the vsize holds the right size
    if (vsize != strlen(val1)) {
	printf("vsize: %d\n", vsize);
	assert(false);
    }

    // make sure the returned value is right
    if (strcmp(ret_val, val1)) {
	printf("%s\n", ret_val);
	assert(false);
    }

    vsize = 0;
    bzero(ret_val, sizeof(ret_val));
    rc = db.get(key1, strlen(key1), ret_val, &vsize);

    // make sure that the db tells us how big a buffer we need
    if (rc != (int)strlen(val1)) {
	printf("rc: %d\n", rc);
	assert(false);
    }

    vsize = sizeof(ret_val);
    bzero(ret_val, 0);
    
    rc = db.get(key2, strlen(key2), ret_val, &vsize);

    // make sure that the db properly complains about not being
    // able to find key2.
    if (rc != -1) {
	printf("rc: %d\n", rc);
	printf("%s\n", ret_val);
	assert(false);
    }

    printf("passed all tests\n");
    exit(0);

}
