/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#ifndef _H_SPAMCOUNTER_H_
#define _H_SPAMCOUNTER_H_

#include "skiplist.h"
#include "util.h"

// Keep track of a number and the number of times it has been used.
struct cntr_entry {
    u_int32_t       cntr;
    int             reps_left;
    sklist_entry<cntr_entry> link;
};

struct entry_cmp {

    entry_cmp() {}
    int operator() (const u_int32_t& e1, const u_int32_t& e2) const {
	if (e1 < e2)      return -1;
	else if (e1 > e2) return 1;
	else              return 0;
    }
};

class reuse_map_t {

    public:

	void init(u_int32_t low, u_int32_t high, int reuses) {

	    for (u_int32_t i = low; i <= high; i++) {
		cntr_entry* ent = new cntr_entry;
		ent->cntr      = i;
		ent->reps_left = reuses;
		if (!s.insert(ent))
		    assert(false);
	    }
	}

	u_int32_t size() const { return s.size(); }

	// it is an error to call this if this object's size() method
	// returns zero.
	u_int32_t get_spam(u_int32_t low, u_int32_t high) {

	    u_int32_t ret;
	    u_int32_t rnd = unifRand(low, high);
	    cntr_entry* ent;

	    if ( !(ent = s.closestsucc(rnd)) )
		assert(false);

	    // make sure we found the items in the right range.
	    if (ent->cntr > high || ent->cntr < low) {
		ent = s.closestpred(rnd);
		if (ent->cntr > high || ent->cntr < low) {
		    printf("none left between %d and %d. returning %d\n", 
			   low, high, low);
		    return low; 
		}
	    }

	    ret = ent->cntr;

	    if (!--ent->reps_left) {
		s.remove(ent->cntr);
		delete ent;
	    }

	    return ret;
	}


    private:

	typedef skiplist<cntr_entry, u_int32_t, &cntr_entry::cntr, 
	                 &cntr_entry::link, entry_cmp> sklist;

	sklist s;
};


#endif
