/*
 *
 * Copyright (C) 2001 Ion Stoica (istoica@cs.berkeley.edu)
 *
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef INCL_UTIL
#define INCL_UTIL

#include <math.h>

#ifndef MAX_INT
#define MAX_INT 0x7fffffff
#endif

inline double funifRand(double a, double b)
{
  int c;
  double f =  (b - a)*((double)(c = random()))/((double)MAX_INT);
 
  return a + f;
}   

/* return a random integer number in the interval [a, b) */
inline int unifRand(int a, int b)
{
  double f;
#ifdef UNIX
  int c = random();
#else
  int c = rand();
#endif /* UNIX */

  if (c == MAX_INT) c--;
  f = (b - a)*((double)c)/((double)MAX_INT);

  return (int)(a + f);
}


inline double fExp(double mean)
{
  double u;

  while ((u = funifRand(0., 1.)) == 0.0);

  return -mean * log(u);
}   


/* functions in util.c */
void  initRand(unsigned seed);
void  initVecRand();
//double funifRand(double a, double b);
//double fExp(double mean);
//int unifRand(int a, int b);
int    intExp(int mean);
double fPareto(double alpha);
double fParetoMean(double mean);
double funifVecRand(double a, double b);
double fVecExp(double mean);
double fVecParetoMean(double mean);
void panic(char *str);

#endif /* INCL_UTIL */
