/*
 * Copyright (C) 2005-2006 Michael Walfish
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "u.h"
#include "send.h"
#include "util2.h"

inline char
nibble2hexchar(char c)
{
    assert(c < 16 && c >= 0);
    if (c >= 0 && c <= 9)  return c + '0';
    else                   return c + 'a' - 10;
}

void
hexdump(const char* buf, int len)
{
    for (int i = 0; i < len; i++) 
	printf("%c%c", nibble2hexchar((buf[i]&0xf0) >> 4),
		       nibble2hexchar(buf[i]&0x0f));
    printf("\n");
}

inline void
hash_to_hex(const uint8* hash)
{
    char hex[2*KEY_VAL_LEN + 1];
    hex[2*KEY_VAL_LEN] = '\0';
    for (int i = 0; i < KEY_VAL_LEN; i++)
	sprintf(hex + 2*i, "%c%c", 
	        nibble2hexchar((hash[i]&0xf0) >> 4),
		nibble2hexchar(hash[i]&0x0f));
    printf("%s\n", hex);
}

inline void
pretty_print(uint64 now, uint64 last, uint64 delay, char* foo)
{
    struct tm* tm1;
    struct timeval tv_now, tv_then;

    us2tv(now, &tv_now);
    us2tv(last+delay, &tv_then);

    tm1 = localtime(reinterpret_cast<time_t*>(&tv_now.tv_sec));
    printf("now: %.2d:%.2d:%.2d.%.6ld. ", 
	   tm1->tm_hour, tm1->tm_min, tm1->tm_sec, 
	   (long int)tv_now.tv_usec);

    tm1 = localtime(reinterpret_cast<time_t*>(&tv_then.tv_sec));
    printf("scheduling %s for %.2d:%.2d:%.2d.%.6ld\n", 
	   foo, tm1->tm_hour, tm1->tm_min, tm1->tm_sec, 
	   (long int)tv_then.tv_usec);
}

inline void
pretty_print_resp(portal_resp* resp)
{
    portal_resp_hdr* hdr = &resp->hdr;
    portal_resp_body* body = &resp->body;
    printf("resp: xid: %d. type: %d. reply_stat: %d. "
           "accept_stat: %d. res: %d\n",
	   ntohl(hdr->xid),
	   ntohl(hdr->type),
	   ntohl(hdr->reply_stat),
	   ntohl(hdr->accept_stat),
	   ntohl(body->res));
}

void 
dump_req(event* ev, portal_req* req, portal* poor)
{
    portal_req_hdr* hdr = &req->hdr;

    if (hdr->proc == PORTAL_TEST_PROC) {
	printf("about to send test to %s:%d. key: ", 
	   poor->name, poor->port);
	hash_to_hex(req->body.test.key);
    } 
   
    if (hdr->proc == PORTAL_SET_PROC) {
	printf("about to send set to %s:%d. key: ",
	   poor->name, poor->port);
	hash_to_hex(req->body.set.key);
    }
}


